#!/usr/bin/env python
'''
Module Name:    MonitorUpgradeSvc.py

Description:    Init script for Python-based Monitor Upgrade Service 

Copyright:      Toshiba Confidential

                OCO Source Materials

                Copyright (C) 2012 Toshiba Global Commerce Solutions, Inc.

                The source code for this program is not published
                or otherwise divested of its trade secrets,
                irrespective of what has been deposited with the
                U. S. Copyright Office.

Initial Author: Francisco Liwa

Function:

Notes:

Dependencies:    None


Restrictions:    None

Created:  3 October, 2012


'''

# MonitorUpgradeSvc- Init script for Python-based Monitor Upgrade Service 
#
# chkconfig:- 99 01
# description: Monitor Upgrade Service for Linux v1.0
# 
### BEGIN INIT INFO 
# Provides: MonitorUpgradeSvc
# Required-Start: 
# Required-Stop: xdm
# Should-Start:                 
# Should-Stop:   xdm                  
# Default-Start:  2 3 5
# Default-Stop:   
# Short-Description: Monitor Upgrade Service
# Description: Monitor Upgrade Service for Linux v1.0
### END INIT INFO

##### Globals #######################
# service name, necessary paths
SERVICE_NAME = 'MonitorUpgradeSvc'
LOCK_NAME = 'TSKDRVSVC_{7833DF4D-8A76-45BB-BF1F-A33843EB217D}'
LOCK_FILE = '/var/lock/subsys/' + LOCK_NAME
SUCCESS_OK = 0
MUS_LOG_FILE_DIR = '/var/log/toshiba/MonitorUpgrade/LOGS/mus/'
MUS_DIR = '/opt/toshiba/MonitorUpgrade/mus/scripts'
#MUS_DIR = '/home/fliwa/projects/tpmus/'


#importing core modules
import sys, os


sys.path.append(MUS_DIR) #TODO: Should change path to target directory
#sys.path.append('/opt/toshiba/MonitorUpgrade/mus/scripts')

    
#importing helper modules s
from initscript import Status as stat
from monitorupgrade import app_main as appMain
from monitorupgrade import TLog as TLOG


#==========================================================================================================
# lock() - Locking mechanism for other process to check like Monitor Inventory driver
#==========================================================================================================
def lock(tlog):
    """
    Create the /var/lock/subsys file
    """ 
    open(LOCK_FILE, 'w').close()
    tlog.logger.info('MonitorUpgradeSvc:lock(): Creating ' + LOCK_FILE + ' lock file')
    
#==========================================================================================================
# locked() - Tells whether the LOCK_FILE exist
#==========================================================================================================
def locked(tlog):
    """
    Return True if the lock file exists
    """
    
    tlog.logger.info('MonitorUpgradeSvc:locked(): ' + LOCK_FILE + ' lock file')
    return os.path.exists(LOCK_FILE)

#==========================================================================================================
# unlock() - Deleting the LOCK_FILE
#==========================================================================================================    
def unlock(tlog):
    """
    Remove the /var/lock/subsys file
    """
    
    tlog.logger.info('MonitorUpgradeSvc:unlock(): Deleting ' + LOCK_FILE + ' lock file')
    os.remove(LOCK_FILE)

#==========================================================================================================
# start() - Starts the MonitorUpgrade Service
#==========================================================================================================
def start():
    """
    Start Toshiba GCS Monitor Upgrade Service
    """
    TLOG.logger.info('                            T O S H I B A   G C S   M O N I T O R   U P G R A D E  S O L U T I O N                                \n\n')
    
    TLOG.logger.info('======================================================== S T A R T  O F  L O G ===================================================\n')
    print "Starting Service...",
    TLOG.logger.info('MonitorUpgradeSvc:start(): Starting Toshiba GCS Monitor Upgrade Service...')
    
    #Let's create the lock first, so that other drivers will know that a task is being performed by MUS
    lock(TLOG)          
    
    #Just print a status to tell that Start is successful
    stat.echo_success()
    
    #Now, Let's run the application
    app = appMain.App()
    app.run()
    
    #if status == SUCCESS_OK:
    #stop()
    """
       Shut everything down, clean up.
    """
    print "Stopping Service...",
    unlock(TLOG)
    stat.echo_success()
    TLOG.logger.info('MonitorUpgradeSvc: Stopping Toshiba GCS Monitor Upgrade Service...\n')
    TLOG.logger.info('=========================================================== E N D  O F  L O G ====================================================\n\n')
    
#==========================================================================================================
# stop() - Stops the MonitorUpgrade Service
#==========================================================================================================
def stop():
    """
    Shut everything down, clean up.
    """
      
    # do stuff
    #unlock()
    #stat.echo_success()
    

#==========================================================================================================
# restart() - Restart the MonitorUpgrade Service
#==========================================================================================================
def restart():
    """
    Stop and then start
    """
    #stop()
    start()

#==========================================================================================================
# status() - Print Status of the MonitorUpgrade Service
#==========================================================================================================
def status():
    """
    Print any relevant status , and return a status code, an integer:
    
    0	      program is running or service is OK
    1	      program is dead and /var/run pid file exists
    2	      program is dead and /var/lock lock file exists
    3	      program is not running
    4	      program or service status is unknown
    5-99	  reserved for future LSB use
    100-149	  reserved for distribution use
    150-199	  reserved for application use
    200-254	  reserved
    
    @see: http://dev.linux-foundation.org/betaspecs/booksets/LSB-Core-generic/LSB-Core-generic/iniscrptact.html
    """
    if not locked(TLOG):
        print "Program isn't running"
        return 3
    else:
        print "Everything is A-OK"
        return 0
    
#==========================================================================================================
# test() -For Unit Testing /Debugging Purpose only
#==========================================================================================================
def test():
    """
    Simple unit testing for the Service
    """
    # This will turn off the system when its complete, 
    # This will also warn the user if they want to continue or not.
    
    ok = raw_input("""
        *******************************
        TESTING MONITOR UPGRADE SERVICE
        *******************************

        This will TURN OFF MonitorUpgradeSvc after all the tests.

        This should only be done for testing and debugging purposes.

        Are you sure you want to do this? [Y/N]: """
        
        ).lower()
    
    if ok != 'y':
        print >> sys.stderr, "Aborting..."
        return

    start()
    # Do stuff to check the start() function     
    #
    # 
    
    print "Verifying Lock File...",
    if os.path.exists(LOCK_FILE):
        stat.echo_success()
    else:
        stat.echo_failure()
        print >> sys.stderr, "ERROR: Lock file was NOT written"
    
    
    
    # we call status a couple of times so we can test if it's returning the right
    # output under different circumstances
    status()
        
    stop()
    # Do stuff to check the stop() function     
    #
    # 
        
    print "Checking lock file after stop...",
    
    if os.path.exists(LOCK_FILE):
        stat.echo_failure()
        print >> sys.stderr, "ERROR: Could not remove lock file"
    else:
        stat.echo_success()
    
    # one more time to see what it looks like when the service off
    status()

#==========================================================================================================
# main() -Main program switch board
#==========================================================================================================

# @TODO: Raise custom exception instead of ValueError, and handle other exceptions better.
 
if __name__ == '__main__':
    
    try:
                   
        # if there's fewer than 2 options on the command line 
        # (sys.argv[0] is the program name)
        if len(sys.argv) == 1:
            raise ValueError; 
                            
        action = str(sys.argv[1]).strip().lower()
        
        if action == 'start':
                            
            start()
            sys.exit(0)
            
        elif action == 'stop':
            stop()          
            stat.echo_success()
            sys.exit(0)
            
        elif action == 'restart' or action == 'force-reload':
            restart()
            sys.exit(0)
            
        elif action == 'status':
            OK = status()
            sys.exit(OK)
            
        elif action == 'test':
            test()
            sys.exit(0)
            
        else:
            raise ValueError
    
    except (SystemExit):
        # calls to sys.exit() raise this error :(
        pass
    
    except (ValueError):
        stat.echo_failure()
        print >> sys.stderr, "Usage: %s [start|stop|status]" % (SERVICE_NAME)
        TLOG.logger.error("Usage: %s [start|stop|status]" % (SERVICE_NAME) )
        # return 2 for "bad command line option"
        sys.exit(2)
    
    except (IOError):
        extype, value = sys.exc_info()[:2]
        stat.echo_failure()
        #print >> sys.stderr, "ERROR: %s (%s)" % (extype, repr(value)) 
        TLOG.logger.error("%s (%s)" % (extype, repr(value) ) )
        # return 2 for "bad command line option"
        sys.exit(2)
    except (RuntimeError):
        extype, value = sys.exc_info()[:2]
        stat.echo_failure()
        #print >> sys.stderr, "ERROR: %s (%s)" % (extype, repr(value)) 
        TLOG.logger.error("%s (%s)" % (extype, repr(value) ) )
        # return 2 for "bad command line option"
        sys.exit(2)     
    
    except:
        # all other exceptions get caught here
        extype, value = sys.exc_info()[:2]
        stat.echo_failure()
        #print >> sys.stderr, "ERROR: %s (%s)" % (extype, repr(value) )
        TLOG.logger.error("%s (%s)" % (extype, repr(value) ) )
        # return 1 for "general error"
        sys.exit(1)

        
