#! /usr/bin/env python
'''
Module Name:    TLog.py

Description:    This is the logger for MUS.

Copyright:      Toshiba Confidential

                OCO Source Materials

                Copyright (C) 2012 Toshiba Global Commerce Solutions, Inc.

                The source code for this program is not published
                or otherwise divested of its trade secrets,
                irrespective of what has been deposited with the
                U. S. Copyright Office.

Initial Author: Francisco Liwa

Function:

Notes:

Dependencies:    None

Restrictions:    None

Created: October 3, 2012

'''
#importing core modules
import os, sys,subprocess

#importing helper modules 
from monitorupgrade import MUSConfigParser
from monitorupgrade import TLog as TLOG

#Globals
FILE_FOUND = 1
FILE_NOT_FOUND = 0
EXIT_SUCCESS = 0
EXIT_FAILURE = 1 
#UPDATE_DONE = 0
DO_REBOOT = False

#Error Codes
ERROR_CODE_DEVICE_NOT_EXIST = 4
ERROR_CODE_OPERATION_FAILED = 11
ERROR_CODE_INVALID_PARAM = 21

#Error Strings
ERROR_STR_DEVICE_NOT_EXIST = 'Device not exist'
ERROR_STR_OPERATION_FAILED = 'Operation failed'
ERROR_STR_INVALID_PARAM = 'Invalid Parameter'
ERROR_STR_Runtime = 'Runtime error'



ELO_CONF = '/etc/opt/toshiba/MonitorUpgrade/CONFIGS/elo/eloConfigOpt.ini'
MUS_CONF = '/etc/opt/toshiba/MonitorUpgrade/CONFIGS/mus/mus_conf.ini'
MUS_LOG_FILE_DIR = '/var/log/toshiba/MonitorUpgrade/mus'

STAGE_DIR =  '/opt/toshiba/MonitorUpgrade/up/4820/stage'
TSK_CONF = '/etc/opt/toshiba/MonitorUpgrade/CONFIGS/mus/update_task/updatetask.conf'
FW_PATH  = '/opt/toshiba/MonitorUpgrade/mus/update_task/fw_temp'
MOD_CONF_BK = '/opt/toshiba/MonitorUpgrade/mus/update_task/modprobe.conf'
MOD_CONF_OR = '/etc/modprobe.conf'

#sampo stuffs here
SAMPO_UTIL = '/opt/toshiba/MonitorUpgrade/op/4820/SAMPO/4820MonUtil'
RECOVERY_FILE_SUSE = '/etc/init.d/after.local'
RECOVERY_STR = '4820MonUtil'

#ELO Stuffs here
ELO_DOWNLOAD = '/opt/toshiba/MonitorUpgrade/op/4820/ELO/EloDownload.sh'
ELO_CONFIG = '/opt/toshiba/MonitorUpgrade/op/4820/ELO/EloConfig.sh'


                
#======================================================== Main Task Manager Class ======================================================        


#==========================================================================================================
'''
TaskMgr - The main class to handle and call other helper classes for individual task
'''
#==========================================================================================================        
class TaskMgr:
    init = False
    fw_path=FW_PATH
    RECOVERY_MODE = False
    DO_REBOOT = False
    
    def __init__(self):
        '''
        Constructor
        '''
        #print "TaskMgr Constructed"
        TLOG.logger.info('TaskMgr Constructed!')

#==========================================================================================================
# initialize() - function to check if task manager has already been initialized
#==========================================================================================================
    def initialize(self):
        
        if (self.init):
            #print'TaskMgr::initialize-Already Initialized'
            TLOG.logger.info('TaskMgr::initialize-Already Initialized')
        else:
            self.init = True
            #print'TaskMgr::initialize-Initialized OK'
            TLOG.logger.info('TaskMgr::initialize-Initialized OK')


#==========================================================================================================
# checkStagingDirectory() - function to check if update task are stage in staging directory
#==========================================================================================================
    def checkStagingDirectory(self):
        '''
        Checking for stage files
        '''
        ret = FILE_NOT_FOUND
        
        TLOG.logger.info('TaskMgr::checkStagingDirectory()-----------------START---------------')
        TLOG.logger.info('TaskMgr::checkStagingDirectory()-Checking Staging directory...')
        
        if ( len(os.listdir( STAGE_DIR )) > 0 ) :
              ret = FILE_FOUND
              print 'TaskMgr::checkStagingDirectory()-Got Staged files !'
              TLOG.logger.info('TaskMgr::checkStagingDirectory()-Got Stage files !')
        
        else:    
              print 'TaskMgr::checkStagingDirectory()-No staged files found'
              TLOG.logger.info('TaskMgr::checkStagingDirectory()')
                    
        TLOG.logger.info('TaskMgr::checkUpdateTaskFile------------------END----------------\n')
        
        return ret
    



#==========================================================================================================
# checkUpdateTaskFile() - function to check if UpdateTask file exist for the InstallHelper task
#==========================================================================================================
    def checkUpdateTaskFile(self):
        '''
        Checking for any available update task 
        '''
        ret = FILE_NOT_FOUND
        
        TLOG.logger.info('TaskMgr::checkUpdateTaskFile-----------------START---------------')
        TLOG.logger.info('TaskMgr::checkUpdateTaskFile-Checking Update task file...')
        
        if doesFileExist(TSK_CONF) == True:
        
            ret = FILE_FOUND
            print 'TaskMgr::checkUpdateTaskFile-Found Update task file...'
            TLOG.logger.info('TaskMgr::checkUpdateTaskFile-Found Update task file...')
        
        else:    
            
            print 'TaskMgr::checkUpdateTaskFile-No Update task file...'
            TLOG.logger.info('TaskMgr::checkUpdateTaskFile-No Update task file...')
                    
        TLOG.logger.info('TaskMgr::checkUpdateTaskFile------------------END----------------\n')
        return ret
    
#==========================================================================================================
# isElousbBlacklisted() - function to check if  elousb is blacklisted or not
#==========================================================================================================
    def isElousbBlacklisted(self):
        '''
        Checking if elousb is blacklisted
        '''
        ret = False
        is_modconf_orig_OK = False
        is_modconf_bkup_OK = False
        
        TLOG.logger.info('TaskMgr::isElousbBlacklisted()-----------------START---------------')
        
        #checking modprobe.conf if it has the blacklist lines
        if doesFileExist(MOD_CONF_OR) == True :
            
            if ( checkString(MOD_CONF_OR,'blacklist elousb') == True ) and ( checkString(MOD_CONF_OR,'install  elousb /bin/true') == True ):
                
                is_modconf_orig_OK = True
                print 'TaskMgr::isElousbBlacklisted()- Found blacklist lines in modprobe.conf'
                TLOG.logger.info('TaskMgr::isElousbBlacklisted()-Found blacklist lines in modprobe.conf')     
        else:
             print 'TaskMgr::isElousbBlacklisted()- /etc/modprobe.conf is not found!'
             TLOG.logger.info('TaskMgr::isElousbBlacklisted()-/etc/modprobe.conf is not found!')
              
        #make sure the backup modprobe.conf has no blacklist lines
        if doesFileExist(MOD_CONF_BK) == True :
            
            if checkString(MOD_CONF_BK,'elousb') == False :
                
                is_modconf_bkup_OK = True
                print 'TaskMgr::isElousbBlacklisted()- Backup modprobe.conf is OK'
                TLOG.logger.info('TaskMgr::isElousbBlacklisted()-Backup modprobe.conf is OK')
        else:
             print 'TaskMgr::isElousbBlacklisted()- /opt/toshiba/MonitorUpgrade/mus/update_task/modprobe.conf is not found!'
             TLOG.logger.info('TaskMgr::isElousbBlacklisted()-/opt/toshiba/MonitorUpgrade/mus/update_task/modprobe.conf is not found!')
             
        
        if (is_modconf_orig_OK and is_modconf_bkup_OK ):
            ret = True
            print 'TaskMgr::isElousbBlacklisted()- elousb is blacklisted'
            TLOG.logger.info('TaskMgr::isElousbBlacklisted()-elousb is blacklisted')
            
        else:
            print 'TaskMgr::isElousbBlacklisted()- is_modconf_orig_OK = ', is_modconf_orig_OK
            print 'TaskMgr::isElousbBlacklisted()- is_modconf_bkup_OK = ', is_modconf_bkup_OK
            
            TLOG.logger.info('TaskMgr::isElousbBlacklisted()- is_modconf_orig_OK = ' + str(is_modconf_orig_OK) )
            TLOG.logger.info('TaskMgr::isElousbBlacklisted()- is_modconf_bkup_OK = ' + str(is_modconf_bkup_OK) )
        
        
        TLOG.logger.info('TaskMgr::isElousbBlacklisted------------------END----------------\n')
        return ret


#==========================================================================================================
# checkRecoveryFile() - function to check if recovery trigger file exist for the Sampo Recovery task
#==========================================================================================================    
    def checkRecoveryFile(self): 
        
        ret = FILE_NOT_FOUND
        
        TLOG.logger.info('TaskMgr::checkRecoveryFile------------------START---------------')
        TLOG.logger.info('TaskMgr::checkRecoveryFile-Checking Sampo Recovery trigger file...')
        if doesFileExist(RECOVERY_FILE_SUSE) == True:
            ret = FILE_FOUND
            #print 'TaskMgr::checkRecoveryFile-Found Sampo Recovery trigger file...'
            TLOG.logger.info('TaskMgr::checkRecoveryFile-Found Sampo Recovery trigger file...')
        else:    
            print 'TaskMgr::checkRecoveryFile-No Sampo Recovery trigger file...'
            TLOG.logger.info('TaskMgr::checkRecoveryFile-No Sampo Recovery trigger file...')
                    
        TLOG.logger.info('TaskMgr::checkRecoveryFile-------------------END----------------\n')
        
        return ret
        

#==========================================================================================================
# isInRecoveryMode() - Returns the recovery mode status of the system
# if in Recovery mode, our application will not do anything, it will exit and let SAMPO do its recovery thing
# The file "/etc/init.d/after.local" which SAMPO utilizes is a mechanism in SuSe Linux where the system during 
# boot up , will execute whatever command is written in that file after all the run level scripts were executed
# including the MonitorUpgradeSvc init script 
#==========================================================================================================    
    def isInRecoveryMode(self):
        
        TLOG.logger.info('TaskMgr::isInRecoveryMode------------------START---------------') 
        TLOG.logger.info('TaskMgr::isInRecoveryMode-Checking if system is in recovery mode...')
        recovery_mode = False
        if ( self.checkRecoveryFile() == FILE_FOUND ) :    
            
            recovery_file = open(RECOVERY_FILE_SUSE,'r+')
            lines = recovery_file.readlines()
            for line  in lines:
                #print line
                if(RECOVERY_STR in line):                  # We look for the string '4820MonUtil', this signifies that there is a pending recovery task
                    
                    HAS_RECOVERY_STR = True 
                    TLOG.logger.info('TaskMgr::isInRecoveryMode()- Found recovery String!- ' + line) 
                    TLOG.logger.info('TaskMgr::isInRecoveryMode()- Proceed with Sampo Recovery Task')
                    print 'TaskMgr::isInRecoveryMode()- Proceed with Sampo Recovery Task'
                    
                    break
                
                else:
                    HAS_RECOVERY_STR = False
            
            recovery_file.close()
            
            if(HAS_RECOVERY_STR):                
                TLOG.logger.info('TaskMgr::isInRecoveryMode()- In recovery Mode ! ')
                print 'TaskMgr::isInRecoveryMode()- In recovery mode'
                recovery_mode = True
            else:
                TLOG.logger.info('TaskMgr::isInRecoveryMode()- Not in Recovery Mode - Sampo 4820MonUtil recovery string is not in file')
                print 'TaskMgr::isInRecoveryMode()- Not in Recovery Mode - Sampo 4820MonUtil recovery string is not in file'
                recovery_mode =  False
        
        TLOG.logger.info('TaskMgr::isInRecoveryMode-------------------END----------------\n')      
        return recovery_mode
    
    def setRebootFlag(self,reboot_):
        TLOG.logger.info('setRebootFlag()------------------START---------------')
    
        self.DO_REBOOT = reboot_
    
        print 'Reboot Flag : ' ,  self.DO_REBOOT
        TLOG.logger.info('setRebootFlag() - Reboot Flag : ' + str(self.DO_REBOOT) ) 
    
        TLOG.logger.info('setRebootFlag()-------------------END----------------\n')    

    def getRebootFlag(self):
    
        TLOG.logger.info('getRebootFlag()------------------START---------------')    
     
        print 'Reboot Flag : ' ,  self.DO_REBOOT
        TLOG.logger.info('getRebootFlag()-------------------END----------------\n')     
        
        return self.DO_REBOOT

#==========================================================================================================
# performTasks() - The main function to be called by the MUS App/ Service to perform the actual update task
#==========================================================================================================    
    def performTasks(self):
        
        '''
            performTasks Sequence of Tasks
                     
            1. check if in recovery mode
            2. get update_task configuration
        ''' 
        
        TLOG.logger.info('TaskMgr::performTasks()=============================================================================================== S T A R T \n')       
        
        #=======================================================================
        # -This will check first if the system will need to do Sampo Recovery mode        
        # -This will check for the file '/etc/init.d/after.local' -this is where
        #   SAMPO will write it's recovery string
        #=======================================================================
        #if ( self.checkRecoveryFile() == FILE_FOUND ) :
        if ( self.isInRecoveryMode() == True):    
                
                self.RECOVERY_MODE  = True
                TLOG.logger.info('TaskMgr::performTasks()- Calling doSampoVideoFlashRecoveryTask()')
                print 'TaskMgr::performTasks()- Calling doSampoVideoFlashRecoveryTask()'
                #cleanUp(True)
                     
#        elif self.checkStagingDirectory() == FILE_FOUND :
#                    
#            TLOG.logger.info('TaskMgr::performTasks()-Running pre-update task now...')
#            print 'TaskMgr::performTasks()-Running pre-update task now...'
#            cmd = str(STAGE_DIR) + '/pre-update'
#            #process = subprocess.Popen(cmd,shell=True, stdout=subprocess.PIPE)
#            process = subprocess.call( cmd ,shell=True)
#
#            
            
        #=======================================================================
        # -We now check for the file 'updatetask.conf' from the  directory 
        #  '/opt/toshiba/MonitorUpgrade/mus/update_task/' 
        # -This file contains the necessary info that the MUS needs to 
        #  know about the update task
        # -A presence of this file means that there is an update to be done                   
        #=======================================================================
        elif self.checkUpdateTaskFile() == FILE_FOUND :
            
            TLOG.logger.info('TaskMgr::performTasks()- Found an Update task available-checking task file now...')
            
            TLOG.logger.info('TaskMgr::performTasks() -Calling MUSConfigParser() ')
            
            update_config = UpdateTaskConfig()
                       
            task_name = update_config.getTaskName()
            print 'task_name = ' + task_name
            
            #===================================================================
            # #setting the firmware path here; for SAMPO utility-the fw directory is enough
            # #For ELO Download utility ; we need to specify which .HEX file or .TXT file to use
            #===================================================================
            
            fw_path = update_config.getFirmwarePath()
            print 'fw_path = ' + fw_path
            self.fw_path = fw_path
            
            TLOG.logger.info('TaskMgr::performTasks()- TaskName= '+ task_name)
            print 'TaskMgr::performTasks()- TaskName= '+ task_name
            
            #doSampoVideoFlashTask_4820_2Lx5Lx        
            if task_name == 'SampoVideoFlashTask_4820_2Lx5Lx' :
                
                TLOG.logger.info('TaskMgr::performTasks()- Calling doSampoVideoFlashTask_4820_2Lx5Lx()')
                                
                exit_status = self.doSampoVideoFlashTask_4820_2Lx5Lx()
                
                #print "Exit Status is:",exit_status
                TLOG.logger.info('TaskMgr::performTasks()-'+ ' Exit Status is:' + str(exit_status) ) 
                if exit_status != 0:
                    raise RuntimeError("SampoVideoFlashTask_4820_2Lx5Lx Task Failed!-exiting :")
            
            #doSampoVideoFlashTask_4820_21x51x        
            elif task_name == 'SampoVideoFlashTask_4820_21x51x' :
                
                TLOG.logger.info('TaskMgr::performTasks()- Calling doSampoVideoFlashTask_4820_21x51x()')
                                
                exit_status = self.doSampoVideoFlashTask_4820_21x51x()
                
                #print "Exit Status is:",exit_status
                TLOG.logger.info('TaskMgr::performTasks()-'+ ' Exit Status is:' + str(exit_status) )
                
                if exit_status != 0:
                    raise RuntimeError("SampoVideoFlashTask_4820_21x51x Task Failed!-exiting :")    
                
            
            #doEloAnakFlashTask
            elif task_name == 'Elo4820_2Lx5LxFlashTask' :
                 
                TLOG.logger.info('TaskMgr::performTasks()- Calling doElo4820_2Lx5LxFlashTask()')
                if self.isElousbBlacklisted() == True:
                                  
                    exit_status = self.doElo4820_2Lx5LxFlashTask()
                               
                    if exit_status != 0:
                        raise RuntimeError("Elo4820_2Lx5LxFlashTask  Failed!-exiting :" )
                    else:
                        self.setRebootFlag(True)   #Needed to bring back elousb, Crystal Display workaround
                        #UPDATE_DONE = 1
                else:
                     TLOG.logger.info('TaskMgr::performTasks()- WARNING: elousb is not blacklisted, FW will not be loaded!')
            
            # doEloYamiFlashTask
            elif task_name == 'Elo4820_21x51xFlashTask' :
                
                TLOG.logger.info('TaskMgr::performTasks()- Calling doElo4820_21x51xFlashTask()')      
                if self.isElousbBlacklisted() == True:
                    
                    exit_status = self.doElo4820_21x51xFlashTask()

                    if exit_status != 0:
                        raise RuntimeError("Elo4820_21x51xFlashTask Failed!-exiting :" )
                    else:
                        self.setRebootFlag(True)   #Needed to bring back elousb, Crystal Display workaround
                        #UPDATE_DONE = 1
                else:
                     TLOG.logger.info('TaskMgr::performTasks()- WARNING: elousb is not blacklisted, FW will not be loaded!')
            
            # doEloConfigTask
            elif task_name == 'EloConfigTask' :
                
                TLOG.logger.info('TaskMgr::performTasks()- Calling doEloConfigTask()')    
                if self.isElousbBlacklisted() == True:
                    
                    exit_status = self.doEloConfigTask()
                
                    if exit_status != 0:
                        raise RuntimeError("EloConfigTask Task Failed!-exiting :" )
                    else:
                        self.setRebootFlag(True)   #Needed to bring back elousb, Crystal Display workaround
                        #UPDATE_DONE = 1
                else:
                     TLOG.logger.info('TaskMgr::performTasks()- WARNING: elousb is not blacklisted, FW will not be loaded!')
            
            else:
                TLOG.logger.info('TaskMgr::performTasks()- Unknown Task!')       
                 
            
        else:
            #We don't have task available, so just exit from the method and kill the service.
            TLOG.logger.info('TaskMgr::performTasks()- No available task!')       
            
               
        TLOG.logger.info('TaskMgr::performTasks()================================================================================================== E N D \n')            
        
        return EXIT_SUCCESS   #will always return success with or without a task!
            
#==========================================================================================================
# doSampoVideoFlashRecoveryTask() -  function to execute SampoVideoFlashRecoveryTask run method
#==========================================================================================================
    def doSampoVideoFlashRecoveryTask(self):
        
        svfrt = SampoVideoFlashRecoveryTask()       
        result = svfrt.run()     
        return result

#==========================================================================================================
# doSampoVideoFlashTask_4820_2Lx5Lx() -  function to execute SampoVideoFlashTask_4820_2Lx5Lx run method
#==========================================================================================================
    def doSampoVideoFlashTask_4820_2Lx5Lx(self):
    
        svft = SampoVideoFlashTask_4820_2Lx5Lx(self.fw_path)      
        result = svft.run()      
        return result


#==========================================================================================================
# doSampoVideoFlashTask_4820_21x51x() -  function to execute SampoVideoFlashTask_4820_21x51x run method
#==========================================================================================================
    def doSampoVideoFlashTask_4820_21x51x(self):
    
        svft = SampoVideoFlashTask_4820_21x51x(self.fw_path)      
        result = svft.run()      
        return result

    
#==========================================================================================================
# do4820_2Lx5LxFlashTask() -  function to execute Elo4820_2Lx5LxFlashTask run method
#==========================================================================================================        
    def doElo4820_2Lx5LxFlashTask(self):
        
        eaft = Elo4820_2Lx5LxFlashTask(self.fw_path)      
        result = eaft.run()
        return result

#==========================================================================================================
# doElo4820_21x51xFlashTask() -  function to execute Elo4820_21x51xFlashTask run method
#==========================================================================================================        
    def doElo4820_21x51xFlashTask(self):
        
        eyft = Elo4820_21x51xFlashTask(self.fw_path)       
        result  = eyft.run()    
        return result

#==========================================================================================================
# doEloConfigTask() -  function to execute EloConfigTask run method
#==========================================================================================================    
    def doEloConfigTask(self):
        
        eCt = EloConfigTask()      
        result = eCt.run()
        return result

        
        

        
#======================================================== Classes for all Defined Task here! ===============================================

            

#==========================================================================================================
'''
SampoVideoFlashRecoveryTask - Class for the Sampo Recovery Task for Video Flash
'''
#==========================================================================================================
class SampoVideoFlashRecoveryTask:
    def __init__(self):
        '''
        Constructor
        '''
        #print 'SampoVideoFlashRecoveryTask Constructed'
        TLOG.logger.info('SampoVideoFlashRecoveryTask Constructed')
    
          
    def run(self):
        TLOG.logger.info('SampoVideoFlashRecoveryTask::run()-----------------START---------------')
        TLOG.logger.info('#####################################################################################################################################')
        TLOG.logger.info('# SampoVideoFlashRecoveryTask::run() Will now let Sampo 4820MonUtil go to Recovery mode, it will perform its own recovery mechanism.#')
        TLOG.logger.info('# SampoVideoFlashRecoveryTask::run() Please check Sampo log at  /opt/toshiba/MonitorUpgrade/op/4820/SAMPO/4820MonUtil.log           #')
        TLOG.logger.info('# SampoVideoFlashRecoveryTask::run() Recovery will start after this script stops.                                                   #')
        TLOG.logger.info('# SampoVideoFlashRecoveryTask::run() The host system will invoke whatever command is written in file /etc/init.d/after.local        #')      
        TLOG.logger.info('#####################################################################################################################################')
        TLOG.logger.info('SampoVideoFlashRecoveryTask::run()-----------------END-----------------\n')
        return EXIT_SUCCESS
    

#==========================================================================================================
'''
SampoVideoFlashTask_4820_2Lx5Lx - Class for Flashing 4820_2Lx5Lx Video FW Update
'''
#==========================================================================================================    
class SampoVideoFlashTask_4820_2Lx5Lx:
    def __init__(self,path):
        '''
        Constructor
        '''
        
        TLOG.logger.info('SampoVideoFlashTask_4820_2Lx5Lx Constructed')
        self.fw_path = path
         
    def run(self):
        TLOG.logger.info('SampoVideoFlashTask_4820_2Lx5Lx::run() -----------------START---------------')
        sampoCommand = SAMPO_UTIL +  '  -SELECT=ALL  -FW_LOCATION=' + str(self.fw_path) 
       
       
        if doesFileExist(SAMPO_UTIL) != True:
            
            print 'SampoVideoFlashTask_4820_2Lx5Lx::run()-No Sampo Utility found!'
            TLOG.logger.error('SampoVideoFlashTask_4820_2Lx5Lx::run()-No Sampo Utility found!')
            raise IOError('No Sampo Utility found!')
        
        else:
            print "This will take some time,Please wait ..."
            ret = runSubprocess(sampoCommand,'SampoVideoFlashTask_4820_2Lx5Lx') 
        
        if ret != EXIT_SUCCESS:
            
            return ret #return immediately
        
        else:
            print 'SampoVideoFlashTask_4820_2Lx5Lx::run() - Task Completed! - Please check log file at '  + MUS_LOG_FILE_DIR         
            TLOG.logger.info('SampoVideoFlashTask_4820_2Lx5Lx::run() -----------------END-----------------\n')
        
            return ret
    
#==========================================================================================================

#==========================================================================================================
'''
SampoVideoFlashTask_4820_21x51x - Class for Flashing 4820_21x51x Video FW Update
'''
#==========================================================================================================    
class SampoVideoFlashTask_4820_21x51x:
    def __init__(self,path):
        '''
        Constructor
        '''
        
        TLOG.logger.info('SampoVideoFlashTask_4820_21x51x Constructed')
        self.fw_path = path
         
    def run(self):
        TLOG.logger.info('SampoVideoFlashTask_4820_21x51x::run() -----------------START---------------')
        sampoCommand = SAMPO_UTIL +  '  -SELECT=ALL  -FW_LOCATION=' + str(self.fw_path) 
       
       
        if doesFileExist(SAMPO_UTIL) != True:
            
            print 'SampoVideoFlashTask_4820_21x51x::run()-No Sampo Utility found!'
            TLOG.logger.error('SampoVideoFlashTask_4820_21x51x::run()-No Sampo Utility found!')
            raise IOError('No Sampo Utility found!')
        
        else:
            print "This will take some time,Please wait ..."
            ret = runSubprocess(sampoCommand,'SampoVideoFlashTask_4820_21x51x') 
        
        if ret != EXIT_SUCCESS:
            
            return ret #return immediately
        
        else:
            print 'SampoVideoFlashTask_4820_21x51x::run() - Task Completed! - Please check log file at '  + MUS_LOG_FILE_DIR         
            TLOG.logger.info('SampoVideoFlashTask_4820_21x51x::run() -----------------END-----------------\n')
        
            return ret
    
#==========================================================================================================
'''
Elo4820_2Lx5LxFlashTask - Class for Flashing ELO 4820_2Lx5Lx Touch FW Update
'''
#==========================================================================================================
class Elo4820_2Lx5LxFlashTask:
    def __init__(self,path):
        '''
        Constructor
        '''
        #print 'Elo4820_2Lx5LxFlashTask Constructed'
        TLOG.logger.info('Elo4820_2Lx5LxFlashTask Constructed')
        self.fw_path = path
        
    def run(self):
        
        TLOG.logger.info('Elo4820_2Lx5LxFlashTask::run() -----------------START---------------')
          
        elo_inventory_cmd = 'sh ' +  ELO_DOWNLOAD + ' -inventory'      
        eloDownloadCmd = 'sh ' +  ELO_DOWNLOAD + ' -select=all -firmware_file=' + str(self.fw_path)
        
        
        print eloDownloadCmd
        if doesFileExist(ELO_DOWNLOAD) != True:
                       
            print 'EloAnakFlashTask::run()-No ELO Download script found!'
            TLOG.logger.error('EloAnakFlashTask::run()-No ELO Download script found!')
            raise IOError('No ELO Download script found!')
        
        else:
            
            print "This will take some time,Please wait ..."
            #elo_inventory_cmd = 'sh ' +  ELO_DOWNLOAD + ' -inventory'
            ret = runSubprocess(elo_inventory_cmd,'ELODOWNLOAD-4820_2Lx5Lx')
            
            if ret != EXIT_SUCCESS:
                return ret #return immediately
            
            #ret = runSubprocess(eloDownloadCmd,'ELODOWNLOAD-4820_2Lx5Lx',str(FW_PATH))
            ret = runSubprocess(eloDownloadCmd,'ELODOWNLOAD-4820_2Lx5Lx')
            
            if ret != EXIT_SUCCESS:
                return ret #return immediately
            
            #Check values if it is updated
            ret = runSubprocess(elo_inventory_cmd,'ELODOWNLOAD-4820_2Lx5Lx')
            
            if ret != EXIT_SUCCESS:
                return ret #return immediately

        
        if ret != EXIT_SUCCESS:
            
            return ret #return immediately
        
        else:
            print 'Elo4820_2Lx5LxFlashTask::run() - Task Completed! - Please check log file at '  + MUS_LOG_FILE_DIR         
            TLOG.logger.info('Elo4820_2Lx5LxFlashTask::run() -----------------END-----------------\n')

            return ret
        

#==========================================================================================================
'''
Elo4820_21x51xFlashTask - Class for Flashing ELO 4820_21x51x Touch FW Update
'''
#==========================================================================================================        
class Elo4820_21x51xFlashTask:
    def __init__(self,path):
        '''
        Constructor
        '''
        #print 'EloYamiFlashTask Constructed'
        TLOG.logger.info('Elo4820_21x51xFlashTask Constructed')
        self.fw_path = path
        
    def run(self):
        
        TLOG.logger.info('Elo4820_21x51xFlashTask::run() -----------------START---------------')
        
        elo_inventory_cmd = 'sh ' +  ELO_DOWNLOAD + ' -inventory'
        eloDownloadCmd = 'sh ' +  ELO_DOWNLOAD + '  -select=all  -firmware_file=' + str(self.fw_path)
        
        
        print eloDownloadCmd
        if doesFileExist(ELO_DOWNLOAD) != True:
                       
            print 'EloAnakFlashTask::run()-No ELO Download script found!'
            TLOG.logger.error('Elo4820_21x51xFlashTask::run()-No ELO Download script found!')
            raise IOError('No ELO Download script found!')
        
        else:
            
            print "This will take some time,Please wait ..."
            #elo_inventory_cmd = 'sh ' +  ELO_DOWNLOAD + ' -inventory'
            ret = runSubprocess(elo_inventory_cmd,'ELODOWNLOAD-4820_21x51x')
            
            if ret != EXIT_SUCCESS:
                return ret #return immediately
            
            #ret = runSubprocess(eloDownloadCmd,'ELODOWNLOAD-4820_21x51x',str(FW_PATH))
            ret = runSubprocess(eloDownloadCmd,'ELODOWNLOAD-4820_21x51x')
        
            if ret != EXIT_SUCCESS:
                return ret #return immediately
            
            #check if values are updated
            ret = runSubprocess(elo_inventory_cmd,'ELODOWNLOAD-4820_21x51x')
            
            if ret != EXIT_SUCCESS:
                return ret #return immediately
            
        if ret != EXIT_SUCCESS:
            
            return ret #return immediately
        else:
            
            print 'Elo4820_21x51xFlashTask::run() - Task Completed! - Please check log file at '  + MUS_LOG_FILE_DIR
            TLOG.logger.info('Elo4820_21x51xFlashTask::run() -----------------END-----------------\n')
 
            return ret
    
   
#==========================================================================================================
'''
EloConfigTask - Do all stuffs here that uses ELO Config Utility 
'''
#==========================================================================================================
class EloConfigTask:
    def __init__(self):
        '''
        Constructor
        '''
        #print 'EloConfigTask Constructed'
        TLOG.logger.info('EloConfigTask Constructed!')
    
    def run(self):
        
        TLOG.logger.info('EloConfigTask::run() -----------------START---------------')
        
        elo_config = EloUtilityConfig()      
              
        TLOG.logger.debug('EloConfigTask::run() -Dumping configuration file... ')
        #elo_config.printConfig()
                       
        TLOG.logger.info('EloConfigTask::run()-================================ Running ELO Config Utility =================================')
        TLOG.logger.info('EloConfigTask::run()- Updating...')
        
        
        #=======================================================================
        # @TODO:  -Find a better way of constructing the command string, for now 
        #            let's just hard code this!
        #        - How to automate what options to update?
        #=======================================================================
        print "This will take some time,Please wait ..."
        
        eloconfig_inventory_cmd = 'sh /opt/toshiba/MonitorUpgrade/op/4820/ELO/EloConfig.sh' + ' -INVENTORY' 
        ret = runSubprocess(eloconfig_inventory_cmd,'ELOCONFIGUTILITY')
        if ret != EXIT_SUCCESS:
            return ret #return immediately
        
        for option in elo_config.options:
            
            if option == 'uit': 
                TLOG.logger.info('EloConfigTask::run()- Updating uit...')
                value = elo_config.getEloOptionValue(option)
                eloConfigCommand = 'sh /opt/toshiba/MonitorUpgrade/op/4820/ELO/EloConfig.sh' + ' -SELECT=ALL -SET_UIT_LEVEL=' + str(value) 
                #ret = runSubprocess(eloConfigCommand,'ELOCONFIGUTILITY')
            
            elif option == 'beepfreq':
                
                TLOG.logger.info('EloConfigTask::run()- Updating beep freq...')
                value = elo_config.getEloOptionValue(option)
                eloConfigCommand = 'sh /opt/toshiba/MonitorUpgrade/op/4820/ELO/EloConfig.sh' + ' -SELECT=ALL -SET_BEEP_FREQUENCY=' + value
                #ret = runSubprocess(eloConfigCommand,'ELOCONFIGUTILITY')
                
            elif option == 'beepmode':
                
                TLOG.logger.info('EloConfigTask::run()- Updating beep mode...')
                value = elo_config.getEloOptionValue(option)
                eloConfigCommand = 'sh /opt/toshiba/MonitorUpgrade/op/4820/ELO/EloConfig.sh' + ' -SELECT=ALL -SET_BEEP_MODE=' + str(value)
                #ret = runSubprocess(eloConfigCommand,'ELOCONFIGUTILITY')
            
            elif option == 'beeptime':
                
                TLOG.logger.info('EloConfigTask::run()- Updating beep duration...')
                value = elo_config.getEloOptionValue(option)
                eloConfigCommand = 'sh /opt/toshiba/MonitorUpgrade/op/4820/ELO/EloConfig.sh' + ' -SELECT=ALL -SET_BEEP_DURATION=' + value
                #ret = runSubprocess(eloConfigCommand,'ELOCONFIGUTILITY')
            
            # @TODO: add more options to update here
                    
            else:            
                eloConfigCommand = 'sh /opt/toshiba/MonitorUpgrade/op/4820/ELO/EloConfig.sh' + '-DESCRIBE' 
                            
            ret = runSubprocess(eloConfigCommand,'ELOCONFIGUTILITY')
            
        #Check if values are updated
        ret = runSubprocess(eloconfig_inventory_cmd,'ELOCONFIGUTILITY')
            
        if ret != EXIT_SUCCESS:
            
            return ret #return immediately
        else:

            print 'EloConfigTask::run() - Task Completed! - Please check log file at '  + MUS_LOG_FILE_DIR
            TLOG.logger.info('EloConfigTask::run() -----------------END-----------------\n')
        
            return ret          
       
    

#===================================================Configuration Classes here!==============================================================



#==========================================================================================================
'''
EloUtilityConfig - Class to handle the contents of eloConfOpt.ini
'''
#==========================================================================================================
class EloUtilityConfig(object):
    def __init__(self):
        '''
        Constructor
        '''
        #print 'EloUtilityConfig Constructed'
        
        TLOG.logger.info('EloUtilityConfig Constructed')
        TLOG.logger.info('EloUtilityConfig -Calling MUSConfigParser() ')
        self.config = MUSConfigParser.MUSConfigParser()
        
        TLOG.logger.info('EloUtilityConfig -Reading configuration file '+ ELO_CONF)
        self.config.readConfig(ELO_CONF)
        
        self.options = self.config.getOptionList() 
        #print self.options
        
    
    def getEloOptionValue(self,option):
        
        config = self.config                         
        TLOG.logger.info('EloUtilityConfig::getEloOptionValue() -' + option + '=' + config.ConfigSectionMap("UserDefinedData")[option] )
        
        return config.ConfigSectionMap("UserDefinedData")[option]
               
    def printConfig(self):
        
        self.config.dump()

#==========================================================================================================
'''
UpdateTaskConfig - Class to handle the contents of updatetask.conf
'''
#==========================================================================================================
class UpdateTaskConfig(object):
    def __init__(self):
        '''
        Constructor
        '''
        #print 'UpdateTaskConfig Constructed'
        
        TLOG.logger.info('UpdateTaskConfig Constructed')
        TLOG.logger.info('UpdateTaskConfig -Calling MUSConfigParser() ')
        self.config = MUSConfigParser.MUSConfigParser()
        
        TLOG.logger.info('UpdateTaskConfig -Reading configuration file '+ TSK_CONF)
        self.config.readConfig(TSK_CONF)
        
         
  
    def getTaskName(self):
        
        config = self.config      
        task_name  =  config.ConfigSectionMap("Task")['taskname']                   
        TLOG.logger.info('UpdateTaskConfig::getTaskName() - task_name = ' + task_name  )
        
        return task_name
    
    def getFirmwarePath(self):
        
        config = self.config      
        fw_path  =  config.ConfigSectionMap("Firmware")['path']                   
        TLOG.logger.info('UpdateTaskConfig::getFirmwarePath() - fw_path = ' + fw_path )
        
        return fw_path
               
    def printConfig(self):
        
        self.config.dump()
        
        
#==========================================================================================================
'''
MusConfig - Class to handle the contents of mus_conf.ini
'''
#==========================================================================================================

class MusConfig(object):
    def __init__(self):
        '''
        Constructor
        '''
        #print 'UpdateTaskConfig Constructed'
        
        TLOG.logger.info('MusConfig Constructed')
        TLOG.logger.info('MusConfig -Calling MUSConfigParser() ')
        self.config = MUSConfigParser.MUSConfigParser()
        
        TLOG.logger.info('MusConfig -Reading configuration file '+ MUS_CONF)
        self.config.readConfig(MUS_CONF)
        
         
  
    def getEloTouchFlashAppPath(self):
        
        config = self.config      
        path  =  config.ConfigSectionMap("EloTouchFlash")['path']                   
        TLOG.logger.info('MusConfig::getEloTouchFlashAppPath() - path = ' + path  )
        
        return path
    
    def getSampoVideoFlashAppPath(self):
        
        config = self.config      
        path  =  config.ConfigSectionMap("SampoVideoFlash")['path']                   
        TLOG.logger.info('MusConfig::getSampoVideoFlashAppPath() - path = ' + path  )
        
        return path
               
    def printConfig(self):
        
        self.config.dump()






#===================================================Helper functions here!==============================================================


#==========================================================================================================
'''
doesFileExist - Returns True if the file exist, False otherwise
'''
#==========================================================================================================

def doesFileExist(filename):
    
    if os.path.exists(filename) and os.path.isfile(filename) and os.access(filename, os.R_OK):
        #print filename + ' exists and is readable'
        TLOG.logger.info('doesFileExist()-Found! :'+ filename)
        return True
    else:
        #print filename  + ' does not exist'
        TLOG.logger.info('doesFileExist()-Not Found! :'+ filename)
        #raise IOError
        return False

#==========================================================================================================
'''
checkString() -returns True if the string to be search in a file exist.
'''
#==========================================================================================================

def checkString(filename,search_string):
    
    found = False
#    print filename + ' exists and is readable'
#    TLOG.logger.info('Found! :'+ filename)  
    modfile=file(filename) 
    for line in modfile:
       if search_string in line:
           print line
           found = True
           break
        
    return found

#==========================================================================================================
'''
runSubprocess - This will call the subprocess module to run the command in the cmd string
'''
#==========================================================================================================
def runSubprocess(cmd,proc,cwd_=None):
    
    global DO_REBOOT
    TLOG.logger.info( str(cmd) )
    #std_err = None
    try:
        process = subprocess.Popen(cmd,shell=True, stdout=subprocess.PIPE,stderr=subprocess.PIPE, cwd = cwd_ )
    
       
        while True:
            line = process.stdout.readline()
            #print line
            if not line:
                break
            TLOG.logger.info('        Subprocess[' + proc + '] OUTPUT-->' + ''  +  line )    
    
            process.wait()
      
        ret = process.returncode
        
        
        if ret != EXIT_SUCCESS:
                  
            TLOG.logger.error('return code:' + str(ret) )
            
            #when the monitor is unplugged during Firmware update.
            if ( ret == ERROR_CODE_DEVICE_NOT_EXIST  ) :  
                print '[TaskMgr::runSubprocess()] Error:' + ERROR_STR_Runtime
                raise RuntimeError('Device not exist')
            
            elif (ret == ERROR_CODE_OPERATION_FAILED ) :
                print '[TaskMgr::runSubprocess()] Error:' + ERROR_STR_OPERATION_FAILED
            
            elif (ret == ERROR_CODE_INVALID_PARAM ) :
                print '[TaskMgr::runSubprocess()] Error:' + ERROR_STR_INVALID_PARAM
                    
            elif (proc != 'ELOCONFIGUTILITY'):
                print '[TaskMgr::runSubprocess()] Error:' + str(ret)
                raise IOError(ret)
            #else:
                #raise IOError(ret)
        else:
            
            return EXIT_SUCCESS
    
    except (ValueError):
        
        extype, value = sys.exc_info()[:2]     
        print >> sys.stderr, "Invalid argument in Popen"
        TLOG.logger.error("%s (%s)" % (extype, value) )
        # return 2 for "bad command line option"
        sys.exit(2)
    
    except (IOError):
        
        extype, value = sys.exc_info()[:2]     
        #print >> sys.stderr, "Error in process: %s" % proc 
        TLOG.logger.error("TaskMgr::runSubprocess()-[Error in process: %s] %s (s%)" % ( proc,repr(extype), repr(value) ) )
        DO_REBOOT = True
        cleanUp(False)
        
        return EXIT_FAILURE
        
    except (RuntimeError):      
       
        value = sys.exc_info()[:2]
        #print value
        #print >> sys.stderr, "TaskMgr::runSubprocess() - Error in process: %s" % proc 
        TLOG.logger.error("TaskMgr::runSubprocess()-[RuntimeError in process: [%s] [%s]" % ( proc, repr(value) ) )
        
        if ( proc == 'SampoVideoFlashTask_4820_2Lx5Lx' or proc == 'SampoVideoFlashTask_4820_21x51x' ):
            cleanUp(True)       # Sets the recovery value to True
        elif (proc == 'ELODOWNLOAD-4820_2Lx5Lx' or proc == 'ELODOWNLOAD-4820_21x51x') :
            DO_REBOOT = True
            cleanUp(False)
            
            
        return EXIT_FAILURE   
        
            
    except (OSError):
        
        extype, value = sys.exc_info()[:2]     
        #print >> sys.stderr, "ERROR: %s (%s)" % (extype, value)
        TLOG.logger.error("%s (%s)" % (extype, value) )
        # return 2 for "bad command line option"
        sys.exit(2)
    
    except:
        # all other exceptions get caught here
        extype, value = sys.exc_info()[:2]
        #print >> sys.stderr, "ERROR: %s (%s)" % (extype, repr(value) )
        TLOG.logger.error("%s (%s)" % (extype,  repr(value) ) )
        sys.exit(1)

        
#==========================================================================================================
# cleanUp() -  delete firmware files from the fw_temp directory which MUS checks
#           -  delete updatask.conf configuration file
#==========================================================================================================        
def cleanUp(in_recovery):
       
        #cmd = []
        
        TLOG.logger.info('cleanUp()------------------START---------------')    
        
       
        if (not in_recovery):           
            #print "not in recovery : UPDATE_DONE ", UPDATE_DONE
            #if doesFileExist(TSK_CONF) == True and UPDATE_DONE == 1:
            if doesFileExist(TSK_CONF)== True:
                cmd1 = 'rm' + ' ' + TSK_CONF
                #cmd.append(cmd1)
                TLOG.logger.info(cmd1)
            
                print cmd1
                rem_proc = subprocess.Popen(cmd1,shell=True, stdout=subprocess.PIPE)
                while True:
                    line = rem_proc.stdout.readline()
                    print line
                    if not line:
                        break
                    TLOG.logger.info(line)    
                    rem_proc.wait()
         
             
            if  os.path.exists(FW_PATH) : #Prevents deleting the firmwares needed by Sampo on its recovery
                                   
                cmd2 = 'rm -rf' + ' ' + FW_PATH
                TLOG.logger.info(cmd2)  
                
                print cmd2
                rem_proc = subprocess.Popen(cmd2,shell=True, stdout=subprocess.PIPE)
                while True:
                    line = rem_proc.stdout.readline()
                    print line
                    if not line:
                        break
                    TLOG.logger.info(line)    
                    rem_proc.wait()
                    
            if DO_REBOOT == True :
                print 'rebooting now..'
                proc = subprocess.call('reboot',shell=True)
        
        else:
            #print "in recovery : UPDATE_DONE ", UPDATE_DONE
            #if doesFileExist(TSK_CONF) == True and UPDATE_DONE == 1:
            if doesFileExist(TSK_CONF) == True:
                cmd1 = 'rm' + ' ' + TSK_CONF
                #cmd.append(cmd1)
                TLOG.logger.info(cmd1)
            
                print cmd1
                rem_proc = subprocess.Popen(cmd1,shell=True, stdout=subprocess.PIPE)
                while True:
                    line = rem_proc.stdout.readline()
                    print line
                    if not line:
                        break
                    TLOG.logger.info(line)    
                    rem_proc.wait()
                    
                    
        TLOG.logger.info('cleanUp()-------------------END----------------\n')     
    

        

